-- Dispatch
-- v1.0 by Cronos-Medivh
local addonPrefix = "[Dispatch]"; -- Max 16 Characters
local registerPrefix = C_ChatInfo.RegisterAddonMessagePrefix(addonPrefix);
local macroBody = "#showtooltip SPELLNAME\n/tar PLAYER\n/cast SPELLNAME\n/targetlasttarget";
local addonVersion = "1.0";

-- Resize Raid Warnings to be much larger
local sizer = 1.5;
local font1, size1, flag1 = RaidWarningFrameSlot1:GetFont();
local font2, size2, flag2 = RaidWarningFrameSlot2:GetFont();

RaidWarningFrameSlot1:SetFont(font1,size1*sizer,flag1);
RaidWarningFrameSlot2:SetFont(font2,size2*sizer,flag2);

RaidWarningFrame.timings.RAID_NOTICE_MIN_HEIGHT = RaidWarningFrame.timings.RAID_NOTICE_MIN_HEIGHT*sizer;
RaidWarningFrame.timings.RAID_NOTICE_MAX_HEIGHT = RaidWarningFrame.timings.RAID_NOTICE_MAX_HEIGHT*sizer;

-- Format Messages
local function FormatErrorMessage(message)
	message = "|cffffcc00" .. addonPrefix .. "|r |cffff0000" .. message .. "|r";
	return message;
end

local function ColorTextRed(message)
	message = "|cffff0000" .. message .. "|r";
	return message;
end

local function FormatAlertMessage(message)
	message = "|cffffcc00" .. addonPrefix .. "|r |cff00ff00" .. message .. "|r";
	return message;
end

local function ColorTextGreen(message)
	message = "|cff00ff00" .. message .. "|r";
	return message;
end

local function FormatRequestMessage(message)
	message = "|cffffcc00" .. addonPrefix .. "|r |cff00ccff" .. message .. "|r";
	return message;
end

-- Useful Functions
local function Explode(basestring, parse)
	local parsedString = {};
	for word in basestring:gmatch("([^"..parse.."]+)") do
		parsedString[#parsedString+1] = word;
	end
	return parsedString;
end

local function GetPlayerName()
	local name = UnitName("player");
	local realm = GetRealmName();
	return name .. "-" .. realm;
end

-- Event Handling
local function OnEvent(self, event, arg1, arg2, arg3, arg4, ...)
	if event == "CHAT_MSG_ADDON" and arg1 == addonPrefix then
		local ignoreSender = false;
		for i, name in ipairs(IgnoreList) do
			if name == arg4 then
				ignoreSender = true;
			end
		end
	
		local message = Explode(arg2, ",");
		if message[1] == "RequestSpell" and ignoreSender == false then
			local spell = message[2];
			local spellId = select(7, GetSpellInfo(spell));
			
			if spellId ~= nil then
				startTime, duration, enabled, modRate = GetSpellCooldown(spellId);
				
				if duration == 0 then
					-- Reading Processed Data
					print(FormatRequestMessage(arg4 .. " wants you to use " .. spell .. " on them"));
					RaidNotice_AddMessage(RaidWarningFrame, ColorTextGreen(arg4 .. " has asked you to use " .. spell .. " on them"), ChatTypeInfo["RAID_WARNING"]);
					PlaySound(8959);
				else
					-- Sending Unprocessed Data
					local cooldownLeft = startTime + duration - GetTime();
					cooldownLeft = math.ceil(cooldownLeft);
					C_ChatInfo.SendAddonMessage(addonPrefix, "OnCooldown," .. spell .. "," .. cooldownLeft .. "," .. GetPlayerName(), "WHISPER", arg4);
				end
			else
				-- Sending Unprocessed Data
				C_ChatInfo.SendAddonMessage(addonPrefix, "NoAccess," .. spell .. "," .. GetPlayerName(), "WHISPER", arg4);
			end
			
		elseif message[1] == "OnCooldown" then
			local spell = message[2];
			local cooldownLeft = message[3];
			local player = message[4];
			
			local alert = player .. "'s " .. spell .. " is on cooldown for " .. cooldownLeft .. " more seconds";
			print(FormatErrorMessage(alert));
			RaidNotice_AddMessage(RaidWarningFrame, ColorTextRed(alert), ChatTypeInfo["RAID_WARNING"]);
			PlaySound(8959);
		
		elseif message[1] == "NoAccess" then
			local spell = message[2];
			local player = message[3];
			
			local alert = player .. " does not have access to " .. spell;
			print(FormatErrorMessage(alert));
		
		elseif message[1] ~= "RequestSpell" then
			print(FormatErrorMessage(arg2));
		end
		
	elseif event == "UNIT_SPELLCAST_SENT" then
		if arg2 == GetPlayerName() or arg2 == UnitName("player") or (arg1 == "player" and arg2 == nil) then
			for i, spell in ipairs(TrackedBuffs) do
				if select(1, GetSpellInfo(arg4)) == spell then
					RaidNotice_AddMessage(RaidWarningFrame, ColorTextGreen(spell .. "!"), ChatTypeInfo["RAID_WARNING"]);
					PlaySound(8959);
					print(FormatRequestMessage(spell .. "!"));
					break;
				end
			end
		end
		
	elseif event == "ADDON_LOADED" and arg1 == "Dispatch" then
		-- Loadup Ignore List
		if IgnoreList == nil then
			IgnoreList = {};
		else
			local newTable = {};
			for i, name in ipairs(IgnoreList) do
				if name ~= -1 then
					newTable[#newTable+1] = name;
				end
			end
			IgnoreList = newTable;
		end
		
		-- Loadup Tracked Buffs
		if TrackedBuffs == nil then
			TrackedBuffs = {};
		else
			local newTable = {};
			for i, name in ipairs(TrackedBuffs) do
				if name ~= -1 then
					newTable[#newTable+1] = name;
				end
			end
			TrackedBuffs = newTable;
		end
		
		print(FormatAlertMessage("v" .. addonVersion .. " by Cronos-Medivh. Type /dispatch for a list of available slash commands."));
	end
end

-- Event Registration
local mainFrame = CreateFrame("Frame");
mainFrame:RegisterEvent("CHAT_MSG_ADDON");
mainFrame:RegisterEvent("UNIT_SPELLCAST_SENT");
mainFrame:RegisterEvent("ADDON_LOADED");
mainFrame:SetScript("OnEvent", OnEvent);

-- Commands

-- /dispatch
-- shows all available slash commands
SLASH_HELP1 = "/dispatch";
SlashCmdList["HELP"] = function(message)
	print(FormatAlertMessage("Available slash commands:"));
	print(FormatAlertMessage("/requestspell <Target-Realm> <Spell Name>; sends an alert to the <Target> to cast <Spell Name> (on you if possible)"));
	print(FormatAlertMessage("/ignorerequests <Target-Realm>; no longer accepts requests from the <Target>"));
	print(FormatAlertMessage("/unignorerequests <Target-Realm>; now accepts requests from the <Target>"));
	print(FormatAlertMessage("/displayignoredrequests; displays all the players you are currently ignoring"));
	print(FormatAlertMessage("/createspellmacro <Target> <Spell Name>; creates a macro that casts <Spell Name> on <Target> whenever pressed"));
	print(FormatAlertMessage("/setspellmacrotarget <Target-Realm> <Spell Name>; changes the <Target> of the <Spell Name> macro"));
	print(FormatAlertMessage("/trackbuff <Spell Name>; alerts you whenever <Spell Name> is casted on you"));
	print(FormatAlertMessage("/untrackbuff <Spell>; stops alerting you whenever <Spell Name> is casted on you"));
	print(FormatAlertMessage("/displaytrackedbuffs; displays all the spells that you are tracking"));
end 

-- /requestspell <Target> <Spell Name>
-- requests the <Target> to cast the <Spell Name>
SLASH_REQUESTSPELL1 = "/requestspell";
SlashCmdList["REQUESTSPELL"] = function(message)
	-- Parse Arguments
	local arguments = Explode(message, " ");
	
	local target = arguments[1];
	
	if target ~= nil then
		local spell = arguments[2];
		
		if spell ~= nil then
			for i, index in ipairs(arguments) do
				if i > 2 then
					spell = spell .. " " .. index;
				end
			end
			
			if not C_ChatInfo.SendAddonMessage(addonPrefix, "RequestSpell," .. spell, "WHISPER", target) then
				print(FormatErrorMessage("Failed to send the spell request"));
			else
				print(FormatRequestMessage("Requested " .. target .. " to cast " .. spell));
			end
		else
			print(FormatErrorMessage("Please supply a spell name"));
		end
	else
		print(FormatAlertMessage("Please suppply a target"));
	end
end

-- /ignorerequests <Target-Realm>
-- no longer accepts requests from the target
SLASH_IGNOREREQUESTS1 = "/ignorerequests";
SLASH_IGNOREREQUESTS2 = "/ignorerequest";
SlashCmdList["IGNOREREQUESTS"] = function(message)
	IgnoreList[#IgnoreList+1] = message;
	print(FormatAlertMessage("Now ignoring all requests from " .. message));
end

-- /unignorerequests <Target-Realm>
-- now accepts requests from the target
SLASH_UNIGNOREREQUESTS1 = "/unignorerequests";
SLASH_UNIGNOREREQUESTS2 = "/unignorerequest";
SlashCmdList["UNIGNOREREQUESTS"] = function(message)
	foundPlayer = false;
	for i, name in ipairs(IgnoreList) do
		if name == message then
			foundPlayer = true;
			IgnoreList[i] = -1;
		end
	end
	
	if foundPlayer == true then
		print(FormatAlertMessage("No longer ignoring all requests from " .. message));
	else
		print(FormatErrorMessage("There was a probelm when trying to unignore " .. message .. " for all requests"));
	end
end

-- /displayignoredrequests
-- displays all the players you are currently ignoring
SLASH_DISPLAYIGNOREDREQUESTS1 = "/displayignoredrequests";
SLASH_DISPLAYIGNOREDREQUESTS2 = "/displayignoredrequest";
SlashCmdList["DISPLAYIGNOREDREQUESTS"] = function(message)
	for i, name in ipairs(IgnoreList) do
		if name ~= -1 then
			print(FormatAlertMessage(i .. ". " .. name));
		end
	end
end

-- /createspellmacro <Target> <Spell Name>
-- creates a macro that casts the specified spell on the target
SLASH_CREATESPELLMACRO1 = "/createspellmacro";
SlashCmdList["CREATESPELLMACRO"] = function(message)
	-- Parse Arguments
	local arguments = Explode(message, " ");
	local target = arguments[1];
	
	if target ~= nil then
		if UnitName("player") == target then
			target = GetPlayerName();
		end
	
		local spell = arguments[2];
		
		if spell ~= nil then
			for i, index in ipairs(arguments) do
				if i > 2 then
					spell = spell .. " " .. index;
				end
			end
			
			if IsUsableSpell(spell) then
				local spellId = select(7, GetSpellInfo(spell));
				local spellIcon = GetSpellTexture(spell);
				local body = macroBody:gsub("SPELLNAME", spell);
				body = body:gsub("PLAYER", target);
				
				if GetMacroIndexByName(spellId) == 0 then
					characterSpecific = true;
					enoughSpace = true;
					if select(2, GetNumMacros()) >= 18 then
						characterSpecific = false;
					end
					if select(1, GetNumMacros()) >= 120 and characterSpecific == false then
						enoughSpace = false
					end
					
					if enoughSpace == true then
						local macro = CreateMacro(spellId, spellIcon, body, characterSpecific);
						
						if GetMacroIndexByName(spellId) ~= 0 then
							print(FormatAlertMessage("Your macro has been created"));
						else
							print(FormatErrorMessage("Something went wrong when trying to create your macro"));
						end
					else
						print(FormatErrorMessage("You do not have enough space for this macro, try deleting an unused macro to continue"));
					end
				else
					print(FormatErrorMessage("A macro with the name: " .. spellId .. " already exists"));
				end
			else
				print(FormatErrorMessage("You cannot use that spell"));
			end
		else
			print(FormatErrorMessage("Please supply a spell name"));
		end
	else
		print(FormatAlertMessage("Please suppply a target"));
	end
end

-- /setspellmacrotarget <Target-Realm> <Spell Name>
-- changes the <Target> of the <Spell Name> macro
SLASH_SETSPELLMACROTARGET1 = "/setspellmacrotarget";
SlashCmdList["SETSPELLMACROTARGET"] = function(message)
	local arguments = Explode(message, " ");
	local target = arguments[1];
	
	if target ~= nil then
		if UnitName("player") == target then
			target = GetPlayerName();
		end
	
		local spell = arguments[2];
		
		if spell ~= nil then
			for i, index in ipairs(arguments) do
				if i > 2 then
					spell = spell .. " " .. index;
				end
			end
			
			characterSpecific = true;
			if select(2, GetNumMacros()) >= 18 then
				characterSpecific = false;
			end
			
			local spellId = select(7, GetSpellInfo(spell));
			local spellIcon = GetSpellTexture(spell);
			local body = macroBody:gsub("SPELLNAME", spell);
			body = body:gsub("PLAYER", target);
			
			if GetMacroIndexByName(spellId) ~= 0 then
				EditMacro(GetMacroIndexByName(spellId), spellId, spellIcon, body, 1, characterSpecific);
				print(FormatAlertMessage("The target for the spell: " .. spell .. " has been changed to: " .. target));
			else
				print(FormatErrorMessage("The macro for the spell: " .. spell .. " does not exist"));
			end
		else
			print(FormatErrorMessage("Please supply a spell name"));
		end
	else
		print(FormatAlertMessage("Please suppply a target"));
	end
	
end

-- /trackbuff <Spell>
-- alerts you whenever this spell is casted on you
SLASH_TRACKBUFF1 = "/trackbuff";
SLASH_TRACKBUFF2 = "/trackspell";
SlashCmdList["TRACKBUFF"] = function(message)
	TrackedBuffs[#TrackedBuffs+1] = message;
	print(FormatAlertMessage("Now alerting you whenever " .. message .. " is casted on you."));
end

-- /untrackbuff <Spell>
-- no longer alerts you whenever <Spell> is casted on you
SLASH_UNTRACKBUFF1 = "/untrackbuff";
SLASH_UNTRACKBUFF2 = "/untrackspell";
SlashCmdList["UNTRACKBUFF"] = function(message)
	foundSpell = false;
	for i, spell in ipairs(TrackedBuffs) do
		if spell == message then
			foundSpell = true;
			TrackedBuffs[i] = -1;
		end
	end
	
	if foundSpell == true then
		print(FormatAlertMessage("No longer receving alerts from " .. message));
	else
		print(FormatErrorMessage("There was a probelm when trying to remove alerts from " .. message));
	end
end

-- /displaytrackedbuffs
-- displays all the buffs you are tracking
SLASH_DISPLAYTRACKEDBUFFS1 = "/displaytrackedbuffs";
SLASH_DISPLAYTRACKEDBUFFS2 = "/displaytrackedbuff";
SLASH_DISPLAYTRACKEDBUFFS3 = "/displaytrackedspells";
SLASH_DISPLAYTRACKEDBUFFS4 = "/displaytrackedspell";
SlashCmdList["DISPLAYTRACKEDBUFFS"] = function(message)
	for i, spell in ipairs(TrackedBuffs) do
		if spell ~= -1 then
			print(FormatAlertMessage(i .. ". " .. spell));
		end
	end
end 